var matching = require('../libs/match/match');
var enum_match = require('../const/matching_case');
var slug = require('../libs/match/slug')
var jsonStreet = require('./data-street.json')

// Version Update 1.0.7

module.exports = {
  async generate_regex(input_string) {
    let regex_text = ''
    let regex_string = input_string
    try {
      let regex_list = regex_string.split('|')
      let regex_list_new = []
      for (let item of regex_list)
      {
        regex_list_new.push(slug.stringToSlugStillSpace(item))
      }
      regex_text = regex_list_new.join("|")
    } catch (err) {
      regex_text = input_string
    }
    return regex_text
  },

  async detect(input) {
    let regex_check
    const { address } = input

    let ro = {
      type: 'place',
      status: true,
      is_wrong: false,
      confidence: 0,
      case: enum_match.default,
      details: ''
    }
    //start with Đường|Tổ|Phố|Ấp|Khu Công Nghiệp|Khu Chế Xuất|KCN|KDC|Khu Dân Cư|Khu Đô Thị|KĐT|Khu Tập Thể  => 100% sai
    let regex_text = 'đường |đuong |đương |duong |dương |dường |to |tô |tổ |phố |phô |ấp |âp |ap |kcn |kcx |kdt |kđt |ktt '
    regex_check = address.toLowerCase().match(RegExp(regex_text))
    if (regex_check && regex_check["index"] == 0)
    {
      ro.is_wrong = true
      ro.confidence = 100
      ro.code = enum_match.wrong_2006
      ro.details = 'Start with Đường|duong|Tổ|to|Phố|pho|Ấp|ap|Khu Công Nghiệp|Khu Chế Xuất|KCN|KDC|Khu Dân Cư|Khu Đô Thị|KĐT|Khu Tập Thể|Lô'
      return ro
    }

    // regex_text = await this.generate_regex('duong |to |pho |ap |Khu Công Nghiệp|Khu Chế Xuất|KCN |KDC |Khu Dân Cư|Khu Đô Thị|KĐT |Khu Tập Thể')
    regex_text = await this.generate_regex('Khu Công Nghiệp|Khu Chế Xuất|KCN |KDC |Khu Dân Cư|Khu Đô Thị|KĐT |Khu Tập Thể')
    regex_check = slug.stringToSlug(address).toLowerCase().match(RegExp(regex_text.toLowerCase()))
    if (regex_check && regex_check["index"] == 0)
    {
      ro.is_wrong = true
      ro.confidence = 100
      ro.code = enum_match.wrong_2006
      ro.details = 'Start with Đường|duong|Tổ|to|Phố|pho|Ấp|ap|Khu Công Nghiệp|Khu Chế Xuất|KCN|KDC|Khu Dân Cư|Khu Đô Thị|KĐT|Khu Tập Thể|Lô'
      return ro
    }

    // Start with tên đường
    regex_street = jsonStreet.join('|')
    regex_check = slug.stringToSlugStillCharacter(address.toLowerCase()).match(RegExp(regex_street))
    if (regex_check && regex_check["index"] == 0)
    {
      ro.is_wrong = true
      ro.confidence = 100
      ro.code = enum_match.wrong_2005
      ro.details = 'Start with tên đường'
      return ro
    }

    //contains tổ|ấp|khu phố|khu đô thị|lô|kiot
    regex_check = address.toLowerCase().match(RegExp(' tổ| ấp| khu phố| khu đô thị| lô| kiot|,tổ|,ấp|,khu phố|,khu đô thị|,lô|,kiot'.toLowerCase()))
    if (regex_check && regex_check["index"] > -1)
    {
      ro.is_wrong = true
      ro.confidence = 50
      ro.code = enum_match.wrong_2007 // 402
      ro.details = 'Contains tổ|ấp|khu phố|khu đô thị|lô|kiot'
      return ro
    }

    let checkPlace = matching.split_num_str(address)

    // start with non-numeric character and no street name => 50% sai
    if (!checkPlace.num)
    {
      ro.is_wrong = true
      ro.confidence = 50
      ro.code = enum_match.wrong_2008
      ro.details = 'Địa danh không bắt đầu bằng số nhà'
      return ro
    }

    // contain A|B|C|... after numeric => 50%
    let globalRegex = RegExp('.*[a-zA-Z]+.*', 'g')
    if (globalRegex.test(checkPlace.num))
    {
      ro.is_wrong = true
      ro.confidence = 50
      ro.code = enum_match.wrong_2009
      ro.details = 'Chữ ngay sau số nhà'
      return ro
    }

    // start with "1 "
    if (checkPlace.pure_str.substr(0,2) == "1 ")
    {
      ro.is_wrong = true
      ro.confidence = 50
      ro.code = enum_match.wrong_2010
      ro.details = 'Bắt đầu bằng 1_'
      return ro
    }

    return ro
  }
}