var detector_regex = require('../detector/regex')
var detector_osrm = require('../detector/osrm')
var detector_admin_centroid = require('../detector/admin_centroid')

module.exports = function () {
  _matchPercent = 70
  _nearByMeters = 50
  _osrmURL = ''
  _vnpostURL = ''
  _vnpostKEY = ''
  _pointRegionMiddle = 18.7008462
  _admincentroidURL = ''
  _apikey = ''

  this.Init = (settings) => {
    if (settings.osrm_url) this._osrmURL = settings.osrm_url
    if (settings.vnpost_url) this._vnpostURL = settings.vnpost_url
    if (settings.vnpost_key) this._vnpostKEY = settings.vnpost_key
    if (settings.admin_centroid_url) this._admincentroidURL = settings.admin_centroid_url
    if (settings.api_key) this._apikey = settings.api_key
  }

  this.map_pipeline = (input) => {
    switch(input) {
      case 'regex': return [detector_regex]
      case 'osrm': return [detector_osrm]
      case 'admin_centroid': return [detector_admin_centroid]
      case 'o_p': return [detector_regex, detector_osrm]
      case 'p_o': return [detector_osrm, detector_regex]
    }
  }

  // Logic :
  // 1. Compare trong Order (địa chỉ , vị trí) : tên địa chỉ và tên đường gần nhất => Trả về kết quả Match / Unmatch
  //
  // Input : lat, lng, address
  //
  this.check_address = async (input) => {
    const { address, profile, osrm_address, lat, lng } = input

    let is_wrong = false
    let wrong_details = []

    if (profile) {
      // Init per profile OSRM INIT
      await detector_osrm.init({
        osrmURL: this._osrmURL,
        vnpostURL: this._vnpostURL,
        vnpostKEY: this._vnpostKEY
      })

      // Init admin centroid
      await detector_admin_centroid.init({
        apikey: this._apikey,
        url: this._admincentroidURL
      })

      //Map & Run PIPELINE
      let pipeline = this.map_pipeline(profile)
      for(let pipe of pipeline) {
        let filter = await pipe.detect({address, osrm_address, lat, lng })
        wrong_details.push(filter)
      }
    }

    for(let item of wrong_details) {
      if (item.is_wrong) is_wrong = true

      // Always fill confidence properties
      if (item.confidence == undefined) {
        item.confidence = 0
        if (item.is_wrong == true) item.confidence = 100
      }

      // With OSRM Check, is_wrong FALSE => reset confidence
      if (item.is_wrong == false && item.confidence > 0) {
        item.confidence = 0
      }
    }

    return {
      is_wrong: is_wrong,
      wrong_details: wrong_details
    }
  }
}